@@ -46,17 +46,6 @@ const char menuAccessIndicator[] = "_";
 #endif
 #include <commctrl.h>
 
-// For getcwd
-#ifdef _MSC_VER
-#include <direct.h>
-#endif
-#ifdef __BORLANDC__
-#include <dir.h>
-#endif
-#ifdef __DMC__
-#include <dir.h>
-#endif
-
 const char menuAccessIndicator[] = "&";
 
 #endif
@@ -67,6 +56,8 @@ const char menuAccessIndicator[] = "&";
 #include "Scintilla.h"
 #include "SciLexer.h"
 #include "Extender.h"
+#include "FilePath.h"
+#include "PropSetFile.h"
 #include "SciTEBase.h"
 
 PropSetFile::PropSetFile(bool lowerKeys_) : lowerKeys(lowerKeys_) {}
@@ -95,6 +86,10 @@ static bool GetFullLine(const char *&fpc, int &lenData, char *s, int len) {
 			}
 		} else if ((ch == '\\') && (lenData > 0) && ((*fpc == '\r') || (*fpc == '\n'))) {
 			continuation = true;
+			if ((lenData > 1) && ((*fpc == '\r') && (*(fpc+1) == '\r') || (*fpc == '\n') && (*(fpc+1) == '\n')))
+				continuation = false;
+			else if ((lenData > 2) && ((*fpc == '\r') && (*(fpc+1) == '\n') && (*(fpc+2) == '\n' || *(fpc+2) == '\r')))
+				continuation = false;
 		} else {
 			continuation = false;
 			*s++ = ch;
@@ -114,23 +109,22 @@ static bool IsCommentLine(const char *line) {
 	return (*line == '#');
 }
 
-bool PropSetFile::ReadLine(const char *lineBuffer, bool ifIsTrue, const char *directoryForImports,
-                           SString imports[], int sizeImports) {
+bool PropSetFile::ReadLine(const char *lineBuffer, bool ifIsTrue, FilePath directoryForImports,
+                           FilePath imports[], int sizeImports) {
 	//UnSlash(lineBuffer);
 	if (!IsSpaceOrTab(lineBuffer[0]))    // If clause ends with first non-indented line
 		ifIsTrue = true;
 	if (isprefix(lineBuffer, "if ")) {
 		const char *expr = lineBuffer + strlen("if") + 1;
 		ifIsTrue = GetInt(expr);
-	} else if (isprefix(lineBuffer, "import ") && directoryForImports) {
-		char importPath[1024];
-		strcpy(importPath, directoryForImports);
-		strcat(importPath, lineBuffer + strlen("import") + 1);
-		strcat(importPath, ".properties");
+	} else if (isprefix(lineBuffer, "import ") && directoryForImports.IsSet()) {
+		SString importName(lineBuffer + strlen("import") + 1);
+		importName += ".properties";
+		FilePath importPath(directoryForImports, FilePath(importName.c_str()));
 		if (Read(importPath, directoryForImports, imports, sizeImports)) {
 			if (imports) {
 				for (int i = 0; i < sizeImports; i++) {
-					if (!imports[i].length()) {
+					if (!imports[i].IsSet()) {
 						imports[i] = importPath;
 						break;
 					}
@@ -143,8 +137,8 @@ bool PropSetFile::ReadLine(const char *lineBuffer, bool ifIsTrue, const char *di
 	return ifIsTrue;
 }
 
-void PropSetFile::ReadFromMemory(const char *data, int len, const char *directoryForImports,
-                                 SString imports[], int sizeImports) {
+void PropSetFile::ReadFromMemory(const char *data, int len, FilePath directoryForImports,
+                                 FilePath imports[], int sizeImports) {
 	const char *pd = data;
 	char lineBuffer[60000];
 	bool ifIsTrue = true;
@@ -161,14 +155,19 @@ void PropSetFile::ReadFromMemory(const char *data, int len, const char *director
 	}
 }
 
-bool PropSetFile::Read(const char *filename, const char *directoryForImports,
-                       SString imports[], int sizeImports) {
-	FILE *rcfile = fopen(filename, fileRead);
+bool PropSetFile::Read(FilePath filename, FilePath directoryForImports,
+                       FilePath imports[], int sizeImports) {
+	FILE *rcfile = filename.Open(fileRead);
 	if (rcfile) {
 		char propsData[60000];
 		int lenFile = static_cast<int>(fread(propsData, 1, sizeof(propsData), rcfile));
 		fclose(rcfile);
-		ReadFromMemory(propsData, lenFile, directoryForImports, imports, sizeImports);
+		const char *data = propsData;
+		if (memcmp(data, "\xef\xbb\xbf", 3) == 0) {
+			data += 3;
+			lenFile -= 3;
+		}
+		ReadFromMemory(data, lenFile, directoryForImports, imports, sizeImports);
 		return true;
 	}
 	return false;
@@ -184,18 +183,13 @@ void SciTEBase::SetImportMenu() {
 	for (int i = 0; i < importMax; i++) {
 		DestroyMenuItem(menuOptions, importCmdID + i);
 	}
-	if (importFiles[0][0]) {
+	if (importFiles[0].IsSet()) {
 		for (int stackPos = 0; stackPos < importMax; stackPos++) {
 			int itemID = importCmdID + stackPos;
-			if (importFiles[stackPos][0]) {
+			if (importFiles[stackPos].IsSet()) {
 				SString entry = LocaliseString("Open");
 				entry += " ";
-				const char *cpDirEnd = strrchr(importFiles[stackPos].c_str(), pathSepChar);
-				if (cpDirEnd) {
-					entry += cpDirEnd + 1;
-				} else {
-					entry += importFiles[stackPos];
-				}
+				entry += importFiles[stackPos].Name().AsInternal();
 				SetMenuItem(menuOptions, IMPORT_START + stackPos, itemID, entry.c_str());
 			}
 		}
@@ -205,8 +199,8 @@ void SciTEBase::SetImportMenu() {
 void SciTEBase::ImportMenu(int pos) {
 	//Platform::DebugPrintf("Stack menu %d\n", pos);
 	if (pos >= 0) {
-		if (importFiles[pos][0] != '\0') {
-			Open(importFiles[pos].c_str());
+		if (importFiles[pos].IsSet()) {
+			Open(importFiles[pos]);
 		}
 	}
 }
@@ -255,65 +249,30 @@ void SciTEBase::ReadGlobalPropFile() {
 	for (int stackPos = 0; stackPos < importMax; stackPos++) {
 		importFiles[stackPos] = "";
 	}
-	char propfile[MAX_PATH + 20];
-	char propdir[MAX_PATH + 20];
+
 	propsBase.Clear();
-	if (GetDefaultPropertiesFileName(propfile, propdir, sizeof(propfile))) {
-		strcat(propdir, pathSepString);
-		propsBase.Read(propfile, propdir, importFiles, importMax);
-	}
+	FilePath propfileBase = GetDefaultPropertiesFileName();
+	propsBase.Read(propfileBase, propfileBase.Directory(), importFiles, importMax);
+
 	propsUser.Clear();
-	if (GetUserPropertiesFileName(propfile, propdir, sizeof(propfile))) {
-		strcat(propdir, pathSepString);
-		propsUser.Read(propfile, propdir, importFiles, importMax);
-	}
+	FilePath propfileUser = GetUserPropertiesFileName();
+	propsUser.Read(propfileUser, propfileUser.Directory(), importFiles, importMax);
+
 	if (!localisationRead) {
 		ReadLocalisation();
 	}
 }
 
 void SciTEBase::ReadAbbrevPropFile() {
-	char propfile[MAX_PATH + 20];
-	char propdir[MAX_PATH + 20];
 	propsAbbrev.Clear();
-	if (GetAbbrevPropertiesFileName(propfile, propdir, sizeof(propfile))) {
-		strcat(propdir, pathSepString);
-		propsAbbrev.Read(propfile, propdir, importFiles, importMax);
-	}
-}
-
-void ChopTerminalSlash(char *path) {
-	size_t endOfPath = strlen(path) - 1;
-	if (path[endOfPath] == pathSepChar) {
-		path[endOfPath] = '\0';
-	}
-}
-
-void SciTEBase::GetDocumentDirectory(char *docDir, int len) {
-	if (dirName[0]) {
-		strncpy(docDir, dirName, len);
-		docDir[len - 1] = '\0';
-	} else {
-		getcwd(docDir, len);
-		docDir[len - 1] = '\0';
-		// In Windows, getcwd returns a trailing backslash
-		// when the CWD is at the root of a disk, so remove it
-		ChopTerminalSlash(docDir);
-	}
+	propsAbbrev.Read(pathAbbreviations, pathAbbreviations.Directory(), importFiles, importMax);
 }
 
 void SciTEBase::ReadLocalPropFile() {
-	char propdir[MAX_PATH + 20];
-	GetDocumentDirectory(propdir, sizeof(propdir));
-	char propfile[MAX_PATH + 20];
-	strcpy(propfile, propdir);
-#ifndef __vms
+	FilePath propdir = filePath.Directory();
+	FilePath propfile;
+	propfile.Set(propdir, propFileName);
 
-	strcat(propdir, pathSepString);
-	strcat(propfile, pathSepString);
-#endif
-
-	strcat(propfile, propFileName);
 	propsLocal.Clear();
 	propsLocal.Read(propfile, propdir);
 	//Platform::DebugPrintf("Reading local properties from %s\n", propfile);
@@ -351,7 +310,7 @@ static long ColourFromString(const SString &s) {
 }
 
 long ColourOfProperty(PropSet &props, const char *key, ColourDesired colourDefault) {
-	SString colour = props.Get(key);
+	SString colour = props.GetExpanded(key);
 	if (colour.length()) {
 		return ColourFromString(colour);
 	}
@@ -557,19 +516,23 @@ void LowerCaseString(char *s) {
 }
 
 SString SciTEBase::ExtensionFileName() {
-	if (overrideExtension.length())
-		return overrideExtension;
-	else if (fileName[0]) {
-		// Force extension to lower case
-		char fileNameWithLowerCaseExtension[MAX_PATH];
-		strcpy(fileNameWithLowerCaseExtension, fileName);
-		char *extension = strrchr(fileNameWithLowerCaseExtension, '.');
-		if (extension) {
-			LowerCaseString(extension);
+	if (CurrentBuffer()->overrideExtension.length()) {
+		return CurrentBuffer()->overrideExtension;
+	} else {
+		FilePath name = FileNameExt();
+		if (name.IsSet()) {
+			// Force extension to lower case
+			char fileNameWithLowerCaseExtension[MAX_PATH];
+				strcpy(fileNameWithLowerCaseExtension, name.AsInternal());
+			char *extension = strrchr(fileNameWithLowerCaseExtension, '.');
+			if (extension) {
+				LowerCaseString(extension);
+			}
+			return SString(fileNameWithLowerCaseExtension);
+		} else {
+			return props.Get("default.file.ext");
 		}
-		return SString(fileNameWithLowerCaseExtension);
-	} else
-		return props.Get("default.file.ext");
+	}
 }
 
 void SciTEBase::ForwardPropertyToEditor(const char *key) {
@@ -651,37 +614,132 @@ SString SciTEBase::FindLanguageProperty(const char *pattern, const char *default
  * A list of all the properties that should be forwarded to Scintilla lexers.
  */
 static const char *propertiesToForward[] = {
+//++Autogenerated -- run src/LexGen.py to regenerate
+//**\(\t"\*",\n\)
+	"asp.default.language",
 	"fold",
-	"fold.comment",
-	"fold.compact",
 	"fold.at.else",
+	"fold.comment",
+	"fold.comment.yaml",
+	"fold.compact",
+	"fold.directive",
 	"fold.html",
 	"fold.html.preprocessor",
 	"fold.perl.package",
 	"fold.perl.pod",
 	"fold.preprocessor",
 	"fold.quotes.python",
+	"fold.sql.only.begin",
 	"fold.verilog.flags",
-	"styling.within.preprocessor",
-	"tab.timmy.whinge.level",
-	"asp.default.language",
 	"html.tags.case.sensitive",
+	"lexer.caml.magic",
+	"lexer.metapost.comment.process",
+	"lexer.metapost.interface.default",
+	"lexer.sql.backticks.identifier",
+	"lexer.tex.auto.if",
+	"lexer.tex.comment.process",
+	"lexer.tex.interface.default",
+	"lexer.tex.use.keywords",
+	"nsis.ignorecase",
+	"nsis.uservars",
 	"ps.level",
 	"ps.tokenize",
 	"sql.backslash.escapes",
+	"styling.within.preprocessor",
+	"tab.timmy.whinge.level",
 
-	"lexer.tex.interface.default",
-	"lexer.tex.comment.process",
-	"lexer.tex.auto.if",
-	"lexer.tex.use.keywords",
-	"lexer.metapost.interface.default",
-	"lexer.metapost.comment.process",
-	"nsis.uservars",
-	"nsis.ignorecase",
+//--Autogenerated -- end of automatically generated section
 
 	0,
 };
 
+/* XPM */
+static char *bookmarkBluegem[] = {
+/* width height num_colors chars_per_pixel */
+"    15    15      64            1",
+/* colors */
+"  c none",
+". c #0c0630",
+"# c #8c8a8c",
+"a c #244a84",
+"b c #545254",
+"c c #cccecc",
+"d c #949594",
+"e c #346ab4",
+"f c #242644",
+"g c #3c3e3c",
+"h c #6ca6fc",
+"i c #143789",
+"j c #204990",
+"k c #5c8dec",
+"l c #707070",
+"m c #3c82dc",
+"n c #345db4",
+"o c #619df7",
+"p c #acacac",
+"q c #346ad4",
+"r c #1c3264",
+"s c #174091",
+"t c #5482df",
+"u c #4470c4",
+"v c #2450a0",
+"w c #14162c",
+"x c #5c94f6",
+"y c #b7b8b7",
+"z c #646464",
+"A c #3c68b8",
+"B c #7cb8fc",
+"C c #7c7a7c",
+"D c #3462b9",
+"E c #7c7eac",
+"F c #44464c",
+"G c #a4a4a4",
+"H c #24224c",
+"I c #282668",
+"J c #5c5a8c",
+"K c #7c8ebc",
+"L c #dcd7e4",
+"M c #141244",
+"N c #1c2e5c",
+"O c #24327c",
+"P c #4472cc",
+"Q c #6ca2fc",
+"R c #74b2fc",
+"S c #24367c",
+"T c #b4b2c4",
+"U c #403e58",
+"V c #4c7fd6",
+"W c #24428c",
+"X c #747284",
+"Y c #142e7c",
+"Z c #64a2fc",
+"0 c #3c72dc",
+"1 c #bcbebc",
+"2 c #6c6a6c",
+"3 c #848284",
+"4 c #2c5098",
+"5 c #1c1a1c",
+"6 c #243250",
+"7 c #7cbefc",
+"8 c #d4d2d4",
+/* pixels */
+"    yCbgbCy    ",
+"   #zGGyGGz#   ",
+"  #zXTLLLTXz#  ",
+" p5UJEKKKEJU5p ",
+" lfISa444aSIfl ",
+" wIYij444jsYIw ",
+" .OsvnAAAnvsO. ",
+" MWvDuVVVPDvWM ",
+" HsDPVkxxtPDsH ",
+" UiAtxohZxtuiU ",
+" pNnkQRBRhkDNp ",
+" 1FrqoR7Bo0rF1 ",
+" 8GC6aemea6CG8 ",
+"  cG3l2z2l3Gc  ",
+"    1GdddG1    "
+};
+
 void SciTEBase::ReadProperties() {
 	if (extender)
 		extender->Clear();
@@ -700,11 +758,7 @@ void SciTEBase::ReadProperties() {
 
 	lexLanguage = SendEditor(SCI_GETLEXER);
 
-	if ((lexLanguage == SCLEX_HTML) || (lexLanguage == SCLEX_XML) ||
-	        (lexLanguage == SCLEX_ASP) || (lexLanguage == SCLEX_PHP))
-		SendEditor(SCI_SETSTYLEBITS, 7);
-	else
-		SendEditor(SCI_SETSTYLEBITS, 5);
+	SendEditor(SCI_SETSTYLEBITS, SendEditor(SCI_GETSTYLEBITSNEEDED));
 
 	SendOutput(SCI_SETLEXER, SCLEX_ERRORLIST);
 
@@ -719,13 +773,10 @@ void SciTEBase::ReadProperties() {
 		SendEditorString(SCI_SETKEYWORDS, wl, kw.c_str());
 	}
 
-	char homepath[MAX_PATH + 20];
-	if (GetSciteDefaultHome(homepath, sizeof(homepath))) {
-		props.Set("SciteDefaultHome", homepath);
-	}
-	if (GetSciteUserHome(homepath, sizeof(homepath))) {
-		props.Set("SciteUserHome", homepath);
-	}
+	FilePath homepath = GetSciteDefaultHome();
+	props.Set("SciteDefaultHome", homepath.AsFileSystem());
+	homepath = GetSciteUserHome();
+	props.Set("SciteUserHome", homepath.AsFileSystem());
 
 	for (size_t i=0; propertiesToForward[i]; i++) {
 		ForwardPropertyToEditor(propertiesToForward[i]);
@@ -736,18 +787,26 @@ void SciTEBase::ReadProperties() {
 		ReadAPI(fileNameForExtension);
 		apisFileNames = props.GetNewExpand("api.", fileNameForExtension.c_str());
 	}
+	FilePath fileAbbrev = props.GetNewExpand("abbreviations.", fileNameForExtension.c_str()).c_str();
+	if (!fileAbbrev.IsSet())
+		fileAbbrev = GetAbbrevPropertiesFileName();
+	if (!pathAbbreviations.SameNameAs(fileAbbrev)) {
+		pathAbbreviations = fileAbbrev;
+		ReadAbbrevPropFile();
+	}
 
 	if (!props.GetInt("eol.auto")) {
 		SetEol();
 	}
 
 	codePage = props.GetInt("code.page");
-	if (unicodeMode != uni8Bit) {
+	if (CurrentBuffer()->unicodeMode != uni8Bit) {
 		// Override properties file to ensure Unicode displayed.
 		codePage = SC_CP_UTF8;
 	}
 	SendEditor(SCI_SETCODEPAGE, codePage);
-	SendOutput(SCI_SETCODEPAGE, codePage);
+	int outputCodePage = props.GetInt("output.code.page", codePage);
+	SendOutput(SCI_SETCODEPAGE, outputCodePage);
 
 	characterSet = props.GetInt("character.set", SC_CHARSET_DEFAULT);
 
@@ -777,6 +836,13 @@ void SciTEBase::ReadProperties() {
 	} else {
 		SendEditor(SCI_SETCARETLINEVISIBLE, 0);
 	}
+	SendEditor(SCI_SETCARETLINEBACKALPHA, props.GetInt("caret.line.back.alpha", SC_ALPHA_NOALPHA));
+
+	SString findMark = props.Get("find.mark");
+	if (findMark.length()) {
+		SendEditor(SCI_INDICSETFORE, 2, ColourFromString(findMark));
+		SendEditor(SCI_INDICSETSTYLE, 2, INDIC_ROUNDBOX);
+	}
 
 	SString controlCharSymbol = props.Get("control.char.symbol");
 	if (controlCharSymbol.length()) {
@@ -785,9 +851,6 @@ void SciTEBase::ReadProperties() {
 		SendEditor(SCI_SETCONTROLCHARSYMBOL, 0);
 	}
 
-	SendEditor(SCI_CALLTIPSETBACK,
-	           ColourOfProperty(props, "calltip.back", ColourDesired(0xff, 0xff, 0xff)));
-
 	SString caretPeriod = props.Get("caret.period");
 	if (caretPeriod.length()) {
 		SendEditor(SCI_SETCARETPERIOD, caretPeriod.value());
@@ -833,6 +896,7 @@ void SciTEBase::ReadProperties() {
 		else	// Have to show selection somehow
 			SendChildren(SCI_SETSELBACK, 1, ColourDesired(0xC0, 0xC0, 0xC0).AsLong());
 	}
+	SendEditor(SCI_SETSELALPHA, props.GetInt("selection.alpha", SC_ALPHA_NOALPHA));
 
 	SString foldColour = props.Get("fold.margin.colour");
 	if (foldColour.length()) {
@@ -901,6 +965,7 @@ void SciTEBase::ReadProperties() {
 	if (sval != "")
 		autoCompleteIgnoreCase = sval == "1";
 	SendEditor(SCI_AUTOCSETIGNORECASE, autoCompleteIgnoreCase ? 1 : 0);
+	SendOutput(SCI_AUTOCSETIGNORECASE, 1);
 
 	int autoCChooseSingle = props.GetInt("autocomplete.choose.single");
 	SendEditor(SCI_AUTOCSETCHOOSESINGLE, autoCChooseSingle),
@@ -966,6 +1031,8 @@ void SciTEBase::ReadProperties() {
 	SendEditor(SCI_SETTABINDENTS, props.GetInt("tab.indents", 1));
 	SendEditor(SCI_SETBACKSPACEUNINDENTS, props.GetInt("backspace.unindents", 1));
 
+	SendEditor(SCI_CALLTIPUSESTYLE, 32);
+
 	indentOpening = props.GetInt("indent.opening");
 	indentClosing = props.GetInt("indent.closing");
 	indentMaintain = props.GetNewExpand("indent.maintain.", fileNameForExtension.c_str()).value();
@@ -1109,12 +1176,17 @@ void SciTEBase::ReadProperties() {
 	           ColourOfProperty(props, "bookmark.fore", ColourDesired(0, 0, 0x7f)));
 	SendEditor(SCI_MARKERSETBACK, SciTE_MARKER_BOOKMARK,
 	           ColourOfProperty(props, "bookmark.back", ColourDesired(0x80, 0xff, 0xff)));
+	SendEditor(SCI_MARKERSETALPHA, SciTE_MARKER_BOOKMARK, props.GetInt("bookmark.alpha", SC_ALPHA_NOALPHA));
 	SString bookMarkXPM = props.Get("bookmark.pixmap");
 	if (bookMarkXPM.length()) {
 		SendEditorString(SCI_MARKERDEFINEPIXMAP, SciTE_MARKER_BOOKMARK,
 			bookMarkXPM.c_str());
-	} else {
+	} else if (props.Get("bookmark.fore").length()) {
 		SendEditor(SCI_MARKERDEFINE, SciTE_MARKER_BOOKMARK, SC_MARK_CIRCLE);
+	} else {
+		// No bookmark.fore setting so display default pixmap.
+		SendEditorString(SCI_MARKERDEFINEPIXMAP, SciTE_MARKER_BOOKMARK,
+			reinterpret_cast<char *>(bookmarkBluegem));
 	}
 
 	SendEditor(SCI_SETSCROLLWIDTH, props.GetInt("horizontal.scroll.width", 2000));
@@ -1128,25 +1200,23 @@ void SciTEBase::ReadProperties() {
 	SendEditor(SCI_SETCARETSTICKY, props.GetInt("caret.sticky", 0));
 
 	if (extender) {
-		char defaultDir[MAX_PATH];
-		GetDefaultDirectory(defaultDir, sizeof(defaultDir));
-		char scriptPath[MAX_PATH];
+		FilePath defaultDir = GetDefaultDirectory();
+		FilePath scriptPath;
 
 		// Check for an extension script
 		SString extensionFile = props.GetNewExpand("extension.", fileNameForExtension.c_str());
 		if (extensionFile.length()) {
 			// find file in local directory
-			char docDir[MAX_PATH];
-			GetDocumentDirectory(docDir, sizeof(docDir));
-			if (Exists(docDir, extensionFile.c_str(), scriptPath)) {
+			FilePath docDir = filePath.Directory();
+			if (Exists(docDir.AsInternal(), extensionFile.c_str(), &scriptPath)) {
 				// Found file in document directory
-				extender->Load(scriptPath);
-			} else if (Exists(defaultDir, extensionFile.c_str(), scriptPath)) {
+				extender->Load(scriptPath.AsFileSystem());
+			} else if (Exists(defaultDir.AsInternal(), extensionFile.c_str(), &scriptPath)) {
 				// Found file in global directory
-				extender->Load(scriptPath);
-			} else if (Exists("", extensionFile.c_str(), scriptPath)) {
+				extender->Load(scriptPath.AsFileSystem());
+			} else if (Exists("", extensionFile.c_str(), &scriptPath)) {
 				// Found as completely specified file name
-				extender->Load(scriptPath);
+				extender->Load(scriptPath.AsFileSystem());
 			}
 		}
 	}
@@ -1189,7 +1259,7 @@ void SciTEBase::ReadFontProperties() {
 	SetStyleFor(wOutput, "*");
 	SetStyleFor(wOutput, "errorlist");
 
-	if (useMonoFont) {
+	if (CurrentBuffer()->useMonoFont) {
 		sval = props.GetExpanded("font.monospace");
 		StyleDefinition sd(sval.c_str());
 		for (int style = 0; style <= STYLE_MAX; style++) {
@@ -1283,19 +1353,15 @@ SString SciTEBase::LocaliseMessage(const char *s, const char *param0, const char
 }
 
 void SciTEBase::ReadLocalisation() {
-	char propfile[MAX_PATH + 20];
-	char propdir[MAX_PATH + 20];
 	propsUI.Clear();
 	const char *title = "locale.properties";
 	SString localeProps = props.GetExpanded(title);
 	if (localeProps.length()) {
 		title = localeProps.c_str();
 	}
-	if (GetSciteDefaultHome(propdir, sizeof propdir)
-	    && BuildPath(propfile, propdir, title, sizeof propfile)) {
-		strcat(propdir, pathSepString);
-		propsUI.Read(propfile, propdir, importFiles, importMax);
-	}
+	FilePath propdir = GetSciteDefaultHome();
+	FilePath localePath(propdir, title);
+	propsUI.Read(localePath, propdir, importFiles, importMax);
 	localisationRead = true;
 }
 
@@ -1303,11 +1369,15 @@ void SciTEBase::ReadPropertiesInitial() {
 	SetPropertiesInitial();
 	int sizeHorizontal = props.GetInt("output.horizontal.size", 0);
 	int sizeVertical = props.GetInt("output.vertical.size", 0);
+	int hideOutput = props.GetInt("output.initial.hide", 0);
 	if ((!splitVertical && (sizeVertical > 0) && (heightOutput < sizeVertical)) ||
 		(splitVertical && (sizeHorizontal > 0) && (heightOutput < sizeHorizontal))) {
-		heightOutput = NormaliseSplit(splitVertical ? sizeHorizontal : sizeVertical);
-		SizeSubWindows();
-		Redraw();
+		previousHeightOutput = splitVertical ? sizeHorizontal : sizeVertical;
+		if (!hideOutput) {
+			heightOutput = NormaliseSplit(previousHeightOutput);
+			SizeSubWindows();
+			Redraw();
+		}
 	}
 	ViewWhitespace(props.GetInt("view.whitespace"));
 	SendEditor(SCI_SETINDENTATIONGUIDES, props.GetInt("view.indentation.guides"));
@@ -1317,8 +1387,6 @@ void SciTEBase::ReadPropertiesInitial() {
 	SendEditor(SCI_SETWRAPMODE, wrap ? wrapStyle : SC_WRAP_NONE);
 	SendOutput(SCI_SETWRAPMODE, wrapOutput ? wrapStyle : SC_WRAP_NONE);
 
-	useMonoFont = false;
-
 	SString menuLanguageProp = props.GetNewExpand("menu.language");
 	languageItems = 0;
 	for (unsigned int i = 0; i < menuLanguageProp.length(); i++) {
@@ -1370,64 +1438,39 @@ void SciTEBase::ReadPropertiesInitial() {
 	}
 #endif
 
-	char homepath[MAX_PATH + 20];
-	if (GetSciteDefaultHome(homepath, sizeof(homepath))) {
-		props.Set("SciteDefaultHome", homepath);
-	}
-	if (GetSciteUserHome(homepath, sizeof(homepath))) {
-		props.Set("SciteUserHome", homepath);
-	}
+	FilePath homepath = GetSciteDefaultHome();
+	props.Set("SciteDefaultHome", homepath.AsFileSystem());
+	homepath = GetSciteUserHome();
+	props.Set("SciteUserHome", homepath.AsFileSystem());
 }
 
-bool SciTEBase::GetDefaultPropertiesFileName(char *pathDefaultProps,
-        char *pathDefaultDir, unsigned int lenPath) {
-	if (!GetSciteDefaultHome(pathDefaultDir, lenPath)) {
-		return false;
-	}
-	return BuildPath(pathDefaultProps, pathDefaultDir, propGlobalFileName, lenPath);
+FilePath SciTEBase::GetDefaultPropertiesFileName() {
+	return FilePath(GetSciteDefaultHome(), propGlobalFileName);
 }
 
-bool SciTEBase::GetAbbrevPropertiesFileName(char *pathAbbrevProps,
-        char *pathDefaultDir, unsigned int lenPath) {
-	if (!GetSciteUserHome(pathDefaultDir, lenPath)) {
-		return false;
-	}
-	return BuildPath(pathAbbrevProps, pathDefaultDir, propAbbrevFileName, lenPath);
+FilePath SciTEBase::GetAbbrevPropertiesFileName() {
+	return FilePath(GetSciteUserHome(), propAbbrevFileName);
 }
 
-bool SciTEBase::GetUserPropertiesFileName(char *pathUserProps,
-        char *pathUserDir, unsigned int lenPath) {
-	if (!GetSciteUserHome(pathUserDir, lenPath)) {
-		return false;
-	}
-	return BuildPath(pathUserProps, pathUserDir, propUserFileName, lenPath);
+FilePath SciTEBase::GetUserPropertiesFileName() {
+	return FilePath(GetSciteUserHome(), propUserFileName);
 }
 
 void SciTEBase::OpenProperties(int propsFile) {
-	char propfile[MAX_PATH + 20];
-	char propdir[MAX_PATH + 20];
+	FilePath propfile;
 	if (propsFile == IDM_OPENLOCALPROPERTIES) {
-		GetDocumentDirectory(propfile, sizeof(propfile));
-#ifdef __vms
-
-		strcpy(propfile, VMSToUnixStyle(propfile));
-#endif
-
-		strcat(propfile, pathSepString);
-		strcat(propfile, propFileName);
+		FilePath propdir = filePath.Directory();
+		propfile.Set(propdir, propFileName);
 		Open(propfile, ofQuiet);
 	} else if (propsFile == IDM_OPENUSERPROPERTIES) {
-		if (GetUserPropertiesFileName(propfile, propdir, sizeof(propfile))) {
-			Open(propfile, ofQuiet);
-		}
+		propfile = GetUserPropertiesFileName();
+		Open(propfile, ofQuiet);
 	} else if (propsFile == IDM_OPENABBREVPROPERTIES) {
-		if (GetAbbrevPropertiesFileName(propfile, propdir, sizeof(propfile))) {
-			Open(propfile, ofQuiet);
-		}
+		propfile = pathAbbreviations;
+		Open(propfile, ofQuiet);
 	} else if (propsFile == IDM_OPENGLOBALPROPERTIES) {
-		if (GetDefaultPropertiesFileName(propfile, propdir, sizeof(propfile))) {
-			Open(propfile, ofQuiet);
-		}
+		propfile = GetDefaultPropertiesFileName();
+		Open(propfile, ofQuiet);
 	} else if (propsFile == IDM_OPENLUAEXTERNALFILE) {
 		SString extlua = props.GetExpanded("ext.lua.startup.script");
 		if (extlua.length()) {
@@ -1443,7 +1486,7 @@ int SciTEBase::GetMenuCommandAsInt(SString commandName) {
 		int id;
 	} idmTable[] = {
 //++Autogenerated -- run ../scintilla/src/LexGen.py to regenerate
-//**\(\t\t{"\*",\t\*},\n\)
+//**1:\(\t\t{"\*",\t\*},\n\)
 		{"IDM_MRUFILE",	IDM_MRUFILE},
 		{"IDM_TOOLS",	IDM_TOOLS},
 		{"IDM_BUFFER",	IDM_BUFFER},
@@ -1515,6 +1558,7 @@ int SciTEBase::GetMenuCommandAsInt(SString commandName) {
 		{"IDM_INS_ABBREV",	IDM_INS_ABBREV},
 		{"IDM_JOIN",	IDM_JOIN},
 		{"IDM_SPLIT",	IDM_SPLIT},
+		{"IDM_DUPLICATE",	IDM_DUPLICATE},
 		{"IDM_INCSEARCH",	IDM_INCSEARCH},
 		{"IDM_ENTERSELECTION",	IDM_ENTERSELECTION},
 		{"IDM_PREVMATCHPPC",	IDM_PREVMATCHPPC},
@@ -1573,6 +1617,8 @@ int SciTEBase::GetMenuCommandAsInt(SString commandName) {
 		{"IDM_CLOSEALL",	IDM_CLOSEALL},
 		{"IDM_SAVEALL",	IDM_SAVEALL},
 		{"IDM_BUFFERSEP",	IDM_BUFFERSEP},
+		{"IDM_PREVFILESTACK",	IDM_PREVFILESTACK},
+		{"IDM_NEXTFILESTACK",	IDM_NEXTFILESTACK},
 		{"IDM_HELP",	IDM_HELP},
 		{"IDM_ABOUT",	IDM_ABOUT},
 		{"IDM_HELP_SCITE",	IDM_HELP_SCITE},
