@@ -173,6 +173,8 @@ SciTEWin::SciTEWin(Extension *ext) : SciTEBase(ext) {
 		::FreeResource(handProps);
 	}
 
+	pathAbbreviations = GetAbbrevPropertiesFileName();
+
 	ReadGlobalPropFile();
 	/// Need to copy properties to variables before setting up window
 	SetPropertiesInitial();
@@ -489,7 +491,7 @@ DWORD SciTEWin::ExecuteOne(const Job &jobToRun, bool &seenOutput) {
 			extender->OnExecute(jobToRun.command.c_str());
 
 			if (jobToRun.flags & jobGroupUndo)
-				Platform::SendScintilla(wEditor.GetID(), SCI_BEGINUNDOACTION);
+				Platform::SendScintilla(wEditor.GetID(), SCI_ENDUNDOACTION);
 
 			Redraw();
 			// A Redraw "might" be needed, since Lua and Director
@@ -1446,7 +1448,10 @@ LRESULT SciTEWin::KeyDown(WPARAM wParam) {
 	return 0l;
 }
 
-LRESULT SciTEWin::KeyUp(WPARAM) {
+LRESULT SciTEWin::KeyUp(WPARAM wParam) {
+	if (wParam == VK_CONTROL) {
+		EndStackedTabbing();
+	}
 	return 0l;
 }
 
