@@ -52,12 +52,16 @@
 #include "KeyWords.h"
 #include "Scintilla.h"
 #include "Extender.h"
+#include "FilePath.h"
+#include "PropSetFile.h"
 #include "SciTEBase.h"
 #include "SciTEKeys.h"
 #include "UniqueInstance.h"
 
 const int SCITE_TRAY = WM_APP + 0;
 
+class Dialog;
+
 /** Windows specific stuff.
  **/
 class SciTEWin : public SciTEBase {
@@ -78,8 +82,11 @@ protected:
 	bool staticBuild;
 	int menuSource;
 
+	// Fields also used in tool execution thread
 	HANDLE hWriteSubProcess;
 	DWORD subProcessGroupId;
+	unsigned int win95DeathDelay;
+	int outputScroll;
 
 	HACCEL hAccTable;
 
@@ -103,6 +110,8 @@ protected:
 
 	Window wFindInFiles;
 
+	virtual void ReadProperties();
+
 	virtual void SizeContentWindows();
 	virtual void SizeSubWindows();
 
@@ -120,11 +129,9 @@ protected:
 	void LocaliseControl(HWND w);
 	void LocaliseDialog(HWND wDialog);
 
-	virtual void FixFilePath();
-	virtual void AbsolutePath(char *fullPath, const char *basePath, int size);
 	int DoDialog(HINSTANCE hInst, const char *resName, HWND hWnd, DLGPROC lpProc);
-	virtual bool OpenDialog(const char *filter=NULL);
-	SString ChooseSaveName(const char *title, const char *filter=0, const char *ext=0);
+	virtual bool OpenDialog(FilePath directory, const char *filter);
+	FilePath ChooseSaveName(FilePath directory, const char *title, const char *filter=0, const char *ext=0);
 	virtual bool SaveAsDialog();
 	virtual void SaveACopy();
 	virtual void SaveAsHTML();
@@ -135,6 +142,7 @@ protected:
 	virtual void LoadSessionDialog();
 	virtual void SaveSessionDialog();
 	virtual bool PreOpenCheck(const char *file);
+	virtual bool IsStdinBlocked();
 
 	/// Print the current buffer.
 	virtual void Print(bool showDialog);
@@ -144,6 +152,7 @@ protected:
 	BOOL HandleReplaceCommand(int cmd);
 
 	virtual int WindowMessageBox(Window &w, const SString &msg, int style);
+	virtual void FindMessageBox(const SString &msg, const SString *findItem=0);
 	virtual void AboutDialog();
 	void DropFiles(HDROP hdrop);
 	void MinimizeToTray();
@@ -151,9 +160,9 @@ protected:
 	SString ProcessArgs(const char *cmdLine);
 	virtual void QuitProgram();
 
-	virtual void GetDefaultDirectory(char *directory, size_t size);
-	virtual bool GetSciteDefaultHome(char *path, unsigned int lenPath);
-	virtual bool GetSciteUserHome(char *path, unsigned int lenPath);
+	virtual FilePath GetDefaultDirectory();
+	virtual FilePath GetSciteDefaultHome();
+	virtual FilePath GetSciteUserHome();
 
 	virtual void SetFileProperties(PropSet &ps);
 	virtual void SetStatusBarText(const char *s);
@@ -177,12 +186,13 @@ protected:
 	BOOL ReplaceMessage(HWND hDlg, UINT message, WPARAM wParam);
 	static BOOL CALLBACK ReplaceDlg(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
 	void PerformGrep();
-	void FillCombos(HWND hDlg);
+	void FillCombos(Dialog &dlg);
 	BOOL GrepMessage(HWND hDlg, UINT message, WPARAM wParam);
 	static BOOL CALLBACK GrepDlg(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
 	BOOL IncrementFindMessage(HWND hDlg, UINT message, WPARAM wParam);
 	static BOOL CALLBACK FindIncrementDlg(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
 	virtual void FindIncrement();
+	bool FindReplaceAdvanced();
 	virtual void Find();
 	virtual void FindInFiles();
 	virtual void Replace();
@@ -224,7 +234,7 @@ public:
 	void Run(const char *cmdLine);
 	DWORD ExecuteOne(const Job &jobToRun, bool &seenOutput);
 	void ProcessExecute();
-	void ShellExec(const SString &cmd, const SString &dir);
+	void ShellExec(const SString &cmd, const char *dir);
 	virtual void Execute();
 	virtual void StopExecute();
 	virtual void AddCommand(const SString &cmd, const SString &dir, JobSubsystem jobType, const SString &input = "", int flags=0);
