@@ -207,7 +207,7 @@ bool SciTEWin::OpenDialog(FilePath directory, const char *filter) {
 	bool succeeded = false;
 	char openName[maxBufferSize]; // maximum common dialog buffer size (says mfc..)
 	openName[0] = '\0';
-	
+
 	OPENFILENAMEA ofn = {
 	       sizeof(OPENFILENAME), 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
 	};
@@ -332,7 +332,7 @@ void SciTEWin::SaveAsPDF() {
 }
 
 void SciTEWin::SaveAsTEX() {
-	FilePath path = ChooseSaveName(filePath.Directory(), "Export File As TeX",
+	FilePath path = ChooseSaveName(filePath.Directory(), "Export File As LaTeX",
 	                              "TeX (.tex)\0*.tex\0", ".tex");
 	if (path.IsSet()) {
 		SaveToTEX(path);
@@ -756,11 +756,11 @@ public:
 	}
 
 	// Handle Unicode controls (assume strings to be UTF-8 on Windows NT)
-	
+
 	SString ItemTextU(int id) {
 		SString result = "";
 		char	msz[CTL_TEXT_BUF];
-	
+
 		if (::IsWindowUnicode(Item(id))) {
 			WCHAR wsz[CTL_TEXT_BUF];
 			if (::GetDlgItemTextW(hDlg, id, wsz, CTL_TEXT_BUF)) {
@@ -771,17 +771,17 @@ public:
 			if (::GetDlgItemTextA(hDlg, id, msz, CTL_TEXT_BUF))
 				result = msz;
 		}
-	
+
 		return result;
 	}
-	
+
 	BOOL SetItemTextU(int id, LPCSTR pmsz) {
 		BOOL bSuccess = FALSE;
 		WCHAR wsz[CTL_TEXT_BUF];
-	
+
 		if (!pmsz || *pmsz == 0)
 			return ::SetDlgItemTextA(hDlg, id, "");
-	
+
 		if (::IsWindowUnicode(Item(id))) {
 			if (::MultiByteToWideChar(CP_UTF8, 0, pmsz, -1, wsz, CTL_TEXT_BUF)) {
 				bSuccess = ::SetDlgItemTextW(hDlg, id, wsz);
@@ -789,12 +789,12 @@ public:
 		} else {
 			bSuccess = ::SetDlgItemTextA(hDlg, id, pmsz);
 		}
-	
+
 		return bSuccess;
 	}
 
 	void SetCheck(int id, bool value) {
-		::SendMessage(::GetDlgItem(hDlg, id), BM_SETCHECK, 
+		::SendMessage(::GetDlgItem(hDlg, id), BM_SETCHECK,
 			value ? BST_CHECKED : BST_UNCHECKED, 0);
 	}
 
@@ -844,7 +844,6 @@ BOOL SciTEWin::FindMessage(HWND hDlg, UINT message, WPARAM wParam) {
 	if (WM_SETFONT == message || WM_NCDESTROY == message)
 		return FALSE;
 	Dialog dlg(hDlg);
-	HWND wFindStyle = dlg.Item(IDFINDSTYLE);
 
 	switch (message) {
 
@@ -858,11 +857,13 @@ BOOL SciTEWin::FindMessage(HWND hDlg, UINT message, WPARAM wParam) {
 		dlg.SetCheck(IDREGEXP, regExp);
 		dlg.SetCheck(IDWRAP, wrapFind);
 		dlg.SetCheck(IDUNSLASH, unSlash);
-		dlg.SetCheck(IDFINDSTYLE, findInStyle);
-		dlg.Enable(IDFINDSTYLE, findInStyle);
-		::SendMessage(wFindStyle, EM_LIMITTEXT, 3, 1);
-		::SetDlgItemInt(hDlg, IDFINDSTYLE, SendEditor(SCI_GETSTYLEAT, SendEditor(SCI_GETCURRENTPOS)), FALSE);
 		dlg.SetCheck(reverseFind ? IDDIRECTIONUP : IDDIRECTIONDOWN, true);
+		if (FindReplaceAdvanced()) {
+			dlg.SetCheck(IDFINDSTYLE, findInStyle);
+			dlg.Enable(IDFINDSTYLE, findInStyle);
+			::SendMessage(dlg.Item(IDFINDSTYLE), EM_LIMITTEXT, 3, 1);
+			::SetDlgItemInt(hDlg, IDFINDSTYLE, SendEditor(SCI_GETSTYLEAT, SendEditor(SCI_GETCURRENTPOS)), FALSE);
+		}
 		return TRUE;
 
 	case WM_CLOSE:
@@ -884,8 +885,10 @@ BOOL SciTEWin::FindMessage(HWND hDlg, UINT message, WPARAM wParam) {
 			regExp = dlg.Checked(IDREGEXP);
 			wrapFind = dlg.Checked(IDWRAP);
 			unSlash = dlg.Checked(IDUNSLASH);
-			findInStyle = dlg.Checked(IDFINDINSTYLE);
-			findStyle = atoi(dlg.ItemTextU(IDFINDSTYLE).c_str());
+			if (FindReplaceAdvanced()) {
+				findInStyle = dlg.Checked(IDFINDINSTYLE);
+				findStyle = atoi(dlg.ItemTextU(IDFINDSTYLE).c_str());
+			}
 			reverseFind = dlg.Checked(IDDIRECTIONUP);
 			::EndDialog(hDlg, IDOK);
 			wFindReplace.Destroy();
@@ -895,8 +898,10 @@ BOOL SciTEWin::FindMessage(HWND hDlg, UINT message, WPARAM wParam) {
 			FindNext(reverseFind);
 			return TRUE;
 		} else if (ControlIDOfCommand(wParam) == IDFINDINSTYLE) {
-			findInStyle = dlg.Checked(IDFINDINSTYLE);
-			dlg.Enable(IDFINDSTYLE, findInStyle);
+			if (FindReplaceAdvanced()) {
+				findInStyle = dlg.Checked(IDFINDINSTYLE);
+				dlg.Enable(IDFINDSTYLE, findInStyle);
+			}
 			return TRUE;
 		}
 	}
@@ -923,8 +928,10 @@ BOOL SciTEWin::HandleReplaceCommand(int cmd) {
 		regExp = dlg.Checked(IDREGEXP);
 		wrapFind = dlg.Checked(IDWRAP);
 		unSlash = dlg.Checked(IDUNSLASH);
-		findInStyle = dlg.Checked(IDFINDINSTYLE);
-		findStyle = atoi(dlg.ItemTextU(IDFINDSTYLE).c_str());
+		if (FindReplaceAdvanced()) {
+			findInStyle = dlg.Checked(IDFINDINSTYLE);
+			findStyle = atoi(dlg.ItemTextU(IDFINDSTYLE).c_str());
+		}
 	}
 	if ((cmd == IDREPLACE) || (cmd == IDREPLACEALL) || (cmd == IDREPLACEINSEL) || (cmd == IDREPLACEINBUF)) {
 		replaceWhat = dlg.ItemTextU(IDREPLACEWITH);
@@ -933,14 +940,14 @@ BOOL SciTEWin::HandleReplaceCommand(int cmd) {
 
 	int replacements = 0;
 	if (cmd == IDOK) {
-		FindNext(reverseFind);	// Find next
+		FindNext(false);
 	} else if (cmd == IDREPLACE) {
 		if (havefound) {
 			ReplaceOnce();
 		} else {
 			CharacterRange crange = GetSelection();
 			SetSelection(crange.cpMin, crange.cpMin);
-			FindNext(reverseFind);
+			FindNext(false);
 			if (havefound) {
 				ReplaceOnce();
 			}
@@ -979,9 +986,11 @@ BOOL SciTEWin::ReplaceMessage(HWND hDlg, UINT message, WPARAM wParam) {
 		dlg.SetCheck(IDREGEXP, regExp);
 		dlg.SetCheck(IDWRAP, wrapFind);
 		dlg.SetCheck(IDUNSLASH, unSlash);
-		dlg.SetCheck(IDFINDSTYLE, findInStyle);
-		dlg.Enable(IDFINDSTYLE, findInStyle);
-		::SetDlgItemInt(hDlg, IDFINDSTYLE, SendEditor(SCI_GETSTYLEAT, SendEditor(SCI_GETCURRENTPOS)), FALSE);
+		if (FindReplaceAdvanced()) {
+			dlg.SetCheck(IDFINDSTYLE, findInStyle);
+			dlg.Enable(IDFINDSTYLE, findInStyle);
+			::SetDlgItemInt(hDlg, IDFINDSTYLE, SendEditor(SCI_GETSTYLEAT, SendEditor(SCI_GETCURRENTPOS)), FALSE);
+		}
 		dlg.SetItemText(IDREPLDONE, "0");
 		if (findWhat.length() != 0 && props.GetInt("find.replacewith.focus", 1)) {
 			::SetFocus(wReplaceWith);
@@ -1001,8 +1010,10 @@ BOOL SciTEWin::ReplaceMessage(HWND hDlg, UINT message, WPARAM wParam) {
 			wFindReplace.Destroy();
 			return FALSE;
 		} else if (ControlIDOfCommand(wParam) == IDFINDINSTYLE) {
-			findInStyle = dlg.Checked(IDFINDINSTYLE);
-			dlg.Enable(IDFINDSTYLE, findInStyle);
+			if (FindReplaceAdvanced()) {
+				findInStyle = dlg.Checked(IDFINDINSTYLE);
+				dlg.Enable(IDFINDSTYLE, findInStyle);
+			}
 			return TRUE;
 		} else {
 			return HandleReplaceCommand(ControlIDOfCommand(wParam));
@@ -1018,7 +1029,7 @@ BOOL CALLBACK SciTEWin::ReplaceDlg(HWND hDlg, UINT message, WPARAM wParam, LPARA
 
 BOOL SciTEWin::IncrementFindMessage(HWND hDlg, UINT message, WPARAM wParam) {
 	// Prevent reentrance when setting text
-	static bool entered = false;	
+	static bool entered = false;
 	if (entered)
 		return FALSE;
 
@@ -1141,6 +1152,10 @@ void SciTEWin::FindIncrement() {
 	WindowSetFocus(wEditor);
 }
 
+bool SciTEWin::FindReplaceAdvanced() {
+	return props.GetInt("find.replace.advanced");
+}
+
 void SciTEWin::Find() {
 	if (wFindIncrement.Created())
 		return;
@@ -1157,7 +1172,7 @@ void SciTEWin::Find() {
 		fr.Flags |= FR_DOWN;
 	fr.lpstrFindWhat = const_cast<char *>(findWhat.c_str());
 	fr.wFindWhatLen = static_cast<WORD>(findWhat.length() + 1);
-	int dialog_id = (!props.GetInt("find.replace.advanced") ? IDD_FIND : IDD_FIND_ADV);
+	int dialog_id = FindReplaceAdvanced() ? IDD_FIND_ADV : IDD_FIND;
 
 	if (IsWindowsNT()) {
 		wFindReplace = ::CreateDialogParamW(hInstance,
@@ -1239,7 +1254,7 @@ BOOL SciTEWin::GrepMessage(HWND hDlg, UINT message, WPARAM wParam) {
 		dlg.SetItemText(IDFINDWHAT, props.Get("find.what").c_str());
 		dlg.SetItemText(IDDIRECTORY, props.Get("find.directory").c_str());
 		if (props.GetNewExpand("find.command") == "") {
-			// Empty means use internal that can respond to flags 
+			// Empty means use internal that can respond to flags
 			dlg.SetCheck(IDWHOLEWORD, wholeWord);
 			dlg.SetCheck(IDMATCHCASE, matchCase);
 		} else {
@@ -1352,7 +1367,7 @@ void SciTEWin::FindInFiles() {
 	props.Set("find.what", findWhat.c_str());
 	FilePath findInDir = filePath.Directory();
 	props.Set("find.directory", findInDir.AsFileSystem());
-	wFindInFiles = ::CreateDialogParam(hInstance, "Grep", MainHWND(), 
+	wFindInFiles = ::CreateDialogParam(hInstance, "Grep", MainHWND(),
 		reinterpret_cast<DLGPROC>(GrepDlg), reinterpret_cast<sptr_t>(this));
 	wFindInFiles.Show();
 }
